import signal
import socket
import os
from os.path import abspath, dirname
import threading

from cryptography.fernet import Fernet

HOME = '/'.join(abspath(dirname(__file__)).split('/')[:3])

def signal_handler(sig, frame):
    print("\033[A                             \033[A")
    print("exiting the server...")
    os._exit(1)

def send_messages():
    """
    Sends input to server which
    sends to other clients
    """
    while True:
        msg = input("")
        if msg != "":
            s.send(bytes(msg, "utf-8"))
        print("\033[A                             \033[A")

def receive_messages(username):
    """
    Constantly reveieves data from
    server and prints to console
    """
    while True:
        data = s.recv(1024).decode("utf-8")
        if data[0] == "1":
            user = data[1:].split(": ")[0] + ": "
            message = data[1:][len(user):]
            if user[:-2] != username:
                print(f"\033[1;34m{user}\033[0m{message}")
            else:  # message was sent by user
                print(f"\033[1;32m{user}\033[0m{message}")
        elif data[0] == "2":
            if "joined the server" in data:
                print(f"\033[32m{data[1:]}\033[0m")
            else:
                print(f"\033[31m{data[1:]}\033[0m")


if __name__ == "__main__":
    
    signal.signal(signal.SIGINT, signal_handler)
    
    os.system("stty -echo") 
    password = input("Password: ")
    os.system("stty echo")
    print() 

    with open(f"{HOME}/.chat-app.key", "rb") as f:
        key = f.read()
    
    fernet = Fernet(key)
    
    # read encrypted message
    with open(f'{HOME}/.chat-app-user-secrets', 'rb') as f:
        encrypted = f.read()

    if password == fernet.decrypt(encrypted).decode("utf-8"):
    
        server = input("server: ")
        port = input("port: ")
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((server, int(port)))
        print("\033[1;32mSuccessfully connected to the server!\033[0m")   
        
        username = input("username: ")
        s.send(bytes(username, 'utf-8'))    

        sm = threading.Thread(target=send_messages)
        rm = threading.Thread(target=receive_messages, args=[username])
        sm.start()
        rm.start()
    else:
        print('incorrect password')



from os.path import abspath, dirname
import os
import signal
import socket
import subprocess
import sys
import threading

from cryptography.fernet import Fernet

HOME = '/'.join(abspath(dirname(__file__)).split('/')[:3])

def signal_handler(sig, frame):
    os._exit(1)

# message statuses (each is first byte of message):
# 1 - message received
# 2 - someone joined/left the server

def handle_client_message(clientsocket, address):
    """
    Send recieved message back to 
    client and also print to console
    """
    n_messages = 0
    username = ""
    while True:
        msg = clientsocket.recv(1024)
        if msg != b'':
            if n_messages == 0:
                username = msg.decode("utf-8")
                msg = username + " has joined the server!"
                for c in clientsockets[:-1]:
                    c.send(bytes(msg, "utf-8"))
            else:
                msg = f"1{username}: " + msg.decode("utf-8")
                for c in clientsockets:
                    c.send(bytes(msg, "utf-8"))

            n_messages += 1
        else:
            clientsocket.close()
            clientsockets.remove(clientsocket)
            addresses.remove(address)
            print(f"Connection for {address} has been severed")
            for c in clientsockets:
                c.send(bytes("2" + username + " has left the server", "utf-8"))
            break

def create_connections():
    """
    Constantly look for clients to connect with
    """
    while True:
        clientsocket, address = s.accept()
        print(f"Connection from {address} has been established!")
        clientsockets.append(clientsocket)
        addresses.append(address)
        hcm = threading.Thread(target=handle_client_message, args=(clientsocket, address))
        hcm.start()

if __name__ == "__main__":

    signal.signal(signal.SIGINT, signal_handler)
    
    os.system("stty -echo")
    password = input("Password: ")
    os.system("stty echo")
    print()
    
    with open(f"{HOME}/.chat-app.key", "rb") as f:
        key = f.read()
    
    fernet = Fernet(key)
    
    # read encrypted message
    with open(f'{HOME}/.chat-app-user-secrets', 'rb') as f:
        encrypted = f.read()

    if password == fernet.decrypt(encrypted).decode("utf-8"):

        ip = subprocess.check_output(
                "ifconfig | grep \"...\....\..\....\" | cut -d ' ' -f2",
                shell=True).decode("utf-8")[:-1] 
        port = input("port: ")

        # create socket
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind((ip, int(port)))
        s.listen(5)
        
        print("Server successfully created")
 
        # client info
        clientsockets = []
        addresses = []

        create_connections()
    else:
        print('\033[1;31mincorrect password\033[0m')



import os
from os.path import dirname, abspath, isdir
from shutil import copyfile

from cryptography.fernet import Fernet

CWD = abspath(dirname(__file__)) 
HOME = '/'.join(CWD.split('/')[:3])

if __name__ == "__main__":
    
    # generate encryption key
    key = Fernet.generate_key()
    with open(f'{HOME}/.chat-app.key', 'wb') as f:
        f.write(key)
    
    # get user info
    os.system("stty -echo")
    password = input('Password: ')
    os.system("stty echo")
    print()
   
    bytes_info = password.encode("utf-8")
    fernet = Fernet(key)
    encrypted = fernet.encrypt(bytes_info)

    with open(f'{HOME}/.chat-app-user-secrets', 'wb') as f:
        f.write(encrypted)

    # create ~/bin dir if it doesn't already exist
    bin_dir = f"{HOME}/bin"

    if not isdir(bin_dir):
        os.mkdir(bin_dir)

    # copy python files to bin
    copyfile(CWD + "/client.py", bin_dir + "/client.py")
    copyfile(CWD + "/server.py", bin_dir + "/server.py")
    
    # create executable to run script
    script = """
#!/bin/bash
if [ $1 = "server" ]; then
    python3 ~/bin/server.py
elif [ $1 = "client" ]; then
    python3 ~/bin/client.py
fi
"""
    with open(f"{bin_dir}/chat-app", "w+") as f:
        f.write(script)
    
    # give permission to executable
    os.system("chmod +x ~/bin/chat-app") 
    
